@@ -24,7 +24,9 @@ utils::globalVariables(c("Fraction", "Performance"))
 #' @param event_class A single string. Either "first" or "second" to specify which level of truth to consider as the "event". Default is "first".
 #' @param exclude A character vector specifying the names of the columns to be excluded from the training process.
 #' @param recipe A user-defined \code{recipe} object for custom preprocessing. If provided, internal recipe steps (imputation, encoding, scaling) are skipped.
-#' @param tune_params A list specifying hyperparameter tuning ranges. Default is \code{NULL}.
+#' @param tune_params A named list of tuning ranges for each algorithm and engine
+#'   pair. Example: \code{list(rand_forest = list(ranger = list(mtry = c(1, 3))))}
+#'   will override the defaults for the ranger engine. Default is \code{NULL}.
 #' @param metric The performance metric to optimize during training.
 #' @param algorithm_engines A named list specifying the engine to use for each algorithm.
 #' @param n_cores An integer specifying the number of CPU cores to use for parallel processing. Default is \code{1}.
@@ -44,19 +46,33 @@ utils::globalVariables(c("Fraction", "Performance"))
 #'   Default is \code{"error"}.
 #' @param impute_custom_function A function that takes a data.frame as input and returns an imputed data.frame. Used only if \code{impute_method = "custom"}.
 #' @param encode_categoricals Logical indicating whether to encode categorical variables. Default is \code{TRUE}.
-#' @param scaling_methods Vector of scaling methods to apply. Default is \code{c("center", "scale")}.
+#' @param scaling_methods Vector of scaling methods to apply. Default is \code{c("center", "scale")}. 
+#' @param balance_method Method to handle class imbalance. One of \code{"none"},
+#'   \code{"upsample"}, or \code{"downsample"}. Applied to the training set for
+#'   classification tasks. Default is \code{"none"}.
+#' @param resamples Optional rsample object providing custom resampling splits.
+#'   If supplied, \code{resampling_method}, \code{folds}, and \code{repeats} are
+#'   ignored.
 #' @param summaryFunction A custom summary function for model evaluation. Default is \code{NULL}.
-#' @param use_default_tuning Logical indicating whether to use default tuning grids when \code{tune_params} is \code{NULL}. Default is \code{FALSE}.
-#' @param tuning_strategy A string specifying the tuning strategy. Options might include \code{"grid"}, \code{"bayes"}, or \code{"none"}. Default is \code{"grid"}.
-#' @param tuning_iterations Number of tuning iterations (applicable for Bayesian or other iterative search methods). Default is \code{10}.
+#' @param use_default_tuning Logical; if \code{TRUE} and \code{tune_params} is \code{NULL}, tuning is performed using default grids. Tuning also occurs when custom \code{tune_params} are supplied. When \code{FALSE} and no custom parameters are given, models are fitted once with default settings. Default is \code{FALSE}.
+#' @param tuning_strategy A string specifying the tuning strategy. Must be one of
+#'   \code{"grid"}, \code{"bayes"}, or \code{"none"}. Default is \code{"grid"}.
+#'   If custom \code{tune_params} are provided while \code{tuning_strategy = "none"},
+#'   they will be ignored with a warning.
+#' @param tuning_iterations Number of iterations for Bayesian tuning. Ignored when
+#'   \code{tuning_strategy} is not \code{"bayes"}. Validation of this argument only
+#'   occurs for the Bayesian strategy. Default is \code{10}.
 #' @param early_stopping Logical indicating whether to use early stopping in Bayesian tuning methods (if supported). Default is \code{FALSE}.
 #' @param adaptive Logical indicating whether to use adaptive/racing methods for tuning. Default is \code{FALSE}.
 #' @param learning_curve Logical. If TRUE, generate learning curves (performance vs. training size).
 #' @param seed An integer value specifying the random seed for reproducibility.
+#' @param verbose Logical; if TRUE, prints progress messages during the training
+#'   and evaluation process.
 #' @importFrom magrittr %>%
 #' @importFrom rsample initial_split training testing
 #' @importFrom recipes recipe step_impute_median step_impute_knn step_impute_bag step_naomit step_dummy step_center step_scale prep bake all_numeric_predictors all_predictors all_nominal_predictors all_outcomes step_zv
 #' @importFrom dplyr filter pull rename_with mutate across where select all_of
+#' @importFrom rlang sym
 #' @importFrom stats as.formula complete.cases
 #' @importFrom doFuture registerDoFuture
 #' @importFrom future plan multisession sequential
@@ -73,12 +89,20 @@ utils::globalVariables(c("Fraction", "Performance"))
 #' iris <- iris[iris$Species != "setosa", ]  # Binary classification
 #' iris$Species <- factor(iris$Species)
 #'
-#' # Train models
+#' # Define a custom tuning grid for the ranger engine
+#' tune <- list(
+#'   rand_forest = list(
+#'     ranger = list(mtry = c(1, 3))
+#'   )
+#' )
+#'
+#' # Train models with custom tuning
 #' model <- fastml(
 #'   data = iris,
 #'   label = "Species",
-#'   algorithms = c("rand_forest", "xgboost", "svm_rbf"), algorithm_engines = c(
-#'   list(rand_forest = c("ranger","aorsf", "partykit", "randomForest")))
+#'   algorithms = "rand_forest",
+#'   tune_params = tune,
+#'   use_default_tuning = TRUE
 #' )
 #'
 #' # View model summary
@@ -110,6 +134,8 @@ fastml <- function(data = NULL,
                    impute_custom_function = NULL,
                    encode_categoricals = TRUE,
                    scaling_methods = c("center", "scale"),
+                   balance_method = c("none", "upsample", "downsample"),
+                   resamples = NULL,
                    summaryFunction = NULL,
                    use_default_tuning = FALSE,
                    tuning_strategy = "grid",
@@ -117,11 +143,14 @@ fastml <- function(data = NULL,
                    early_stopping = FALSE,
                    adaptive = FALSE,
                    learning_curve = FALSE,
-                   seed = 123) {
+                   seed = 123,
+                   verbose = FALSE) {
 
   set.seed(seed)
 
   task <- match.arg(task, c("auto", "classification", "regression"))
+  tuning_strategy <- match.arg(tuning_strategy, c("grid", "bayes", "none"))
+  balance_method <- match.arg(balance_method)
 
   # If explicit train/test provided, ensure both are given
   if (!is.null(train_data) || !is.null(test_data)) {
@@ -197,6 +226,7 @@ fastml <- function(data = NULL,
         dplyr::across(where(is.integer), as.numeric)
       )
 
+    if (verbose) message("Splitting data into training and test sets...")
     # Split into train/test
     if (stratify && task == "classification") {
       split <- rsample::initial_split(data, prop = 1 - test_size, strata = label)
@@ -207,6 +237,24 @@ fastml <- function(data = NULL,
     test_data  <- rsample::testing(split)
   }
 
+  if (task == "classification" && balance_method != "none") {
+    label_sym <- rlang::sym(label)
+    class_counts <- table(train_data[[label]])
+    if (balance_method == "upsample") {
+      max_n <- max(class_counts)
+      train_data <- train_data %>%
+        dplyr::group_by(!!label_sym) %>%
+        dplyr::sample_n(max_n, replace = TRUE) %>%
+        dplyr::ungroup()
+    } else if (balance_method == "downsample") {
+      min_n <- min(class_counts)
+      train_data <- train_data %>%
+        dplyr::group_by(!!label_sym) %>%
+        dplyr::sample_n(min_n, replace = FALSE) %>%
+        dplyr::ungroup()
+    }
+  }
+
 
   if (is.null(metric)) {
     metric <- if (task == "classification") "accuracy" else "rmse"
@@ -399,6 +447,7 @@ fastml <- function(data = NULL,
   # Build or use recipe if user hasn't provided it
   ########################################################################
   if (is.null(recipe)) {
+    if (verbose) message("Creating preprocessing recipe...")
     # Build a default recipe, possibly skipping recipe-based imputation if advanced used
     recipe <- recipe(as.formula(paste(label, "~ .")), data = train_data)
 
@@ -468,6 +517,8 @@ fastml <- function(data = NULL,
     }
   }
 
+  if (verbose) message("Training models: ", paste(algorithms, collapse = ", "))
+
   models <- train_models(
     train_data = train_data,
     label = label,
@@ -476,6 +527,7 @@ fastml <- function(data = NULL,
     resampling_method = resampling_method,
     folds = folds,
     repeats = repeats,
+    resamples = resamples,
     tune_params = tune_params,
     metric = metric,
     summaryFunction = summaryFunction,
@@ -497,6 +549,7 @@ fastml <- function(data = NULL,
     stop("No models were successfully trained.")
   }
 
+  if (verbose) message("Evaluating models...")
   eval_output <- evaluate_models(models, train_data, test_data, label, task, metric, event_class)
   performance <- eval_output$performance
   predictions <- eval_output$predictions
@@ -561,7 +614,13 @@ fastml <- function(data = NULL,
   best_model_name <- sapply(best_model_components, function(comp) comp$engine)
   names(best_model_name) <- sapply(best_model_components, function(comp) comp$algo)
 
+  if (verbose) {
+    msg <- paste0(names(best_model_name), " (", best_model_name, ")", collapse = ", ")
+    message("Best model selected: ", msg)
+  }
 
+
+  if (verbose) message("Preparing preprocessing recipe for downstream use...")
   # Now store processed training data for explainability:
   trained_recipe <- prep(recipe, training = train_data, retain = TRUE)
   processed_train_data <- bake(trained_recipe, new_data = NULL)
@@ -572,6 +631,7 @@ fastml <- function(data = NULL,
 
     # Helper function to run the learning curve step for a given fraction
     run_curve <- function(fraction) {
+      if (verbose) message(sprintf("Learning curve: using %.0f%% of training data", fraction * 100))
       set.seed(seed)
 
       # Select training subset based on the fraction value
@@ -592,10 +652,11 @@ fastml <- function(data = NULL,
         label = label,
         task = task,
         algorithms = algorithms,
-        resampling_method = resampling_method,
-        folds = folds,
-        repeats = repeats,
-        tune_params = tune_params,
+       resampling_method = resampling_method,
+       folds = folds,
+       repeats = repeats,
+       resamples = resamples,
+       tune_params = tune_params,
         metric = metric,
         summaryFunction = summaryFunction,
         seed = seed,
@@ -673,5 +734,6 @@ fastml <- function(data = NULL,
     engine_names = engine_names
   )
   class(result) <- "fastml"
+  if (verbose) message("Training complete.")
   return(result)
 }
