@@ -10,14 +10,25 @@
 #' @param resampling_method Resampling method for cross-validation (e.g., "cv", "repeatedcv", "boot", "none").
 #' @param folds Number of folds for cross-validation.
 #' @param repeats Number of times to repeat cross-validation (only applicable for methods like "repeatedcv").
-#' @param tune_params List of hyperparameter tuning ranges.
+#' @param resamples Optional rsample object. If provided, custom resampling splits
+#'   will be used instead of those created internally.
+#' @param tune_params A named list of tuning ranges. For each algorithm, supply a
+#'   list of engine-specific parameter values, e.g.
+#'   \code{list(rand_forest = list(ranger = list(mtry = c(1, 3)))).}
 #' @param metric The performance metric to optimize.
 #' @param summaryFunction A custom summary function for model evaluation. Default is \code{NULL}.
 #' @param seed An integer value specifying the random seed for reproducibility.
 #' @param recipe A recipe object for preprocessing.
-#' @param use_default_tuning Logical indicating whether to use default tuning grids when \code{tune_params} is \code{NULL}.
-#' @param tuning_strategy A string specifying the tuning strategy ("grid", "bayes", or "none"), possibly with adaptive methods.
-#' @param tuning_iterations Number of iterations for iterative tuning methods.
+#' @param use_default_tuning Logical; if \code{TRUE} and \code{tune_params} is \code{NULL}, tuning is performed using default grids. Tuning also occurs when custom \code{tune_params} are supplied. When \code{FALSE} and no custom parameters are given, the model is fitted once with default settings.
+#' @param tuning_strategy A string specifying the tuning strategy. Must be one of
+#'   \code{"grid"}, \code{"bayes"}, or \code{"none"}. Adaptive methods may be
+#'   used with \code{"grid"}. If \code{"none"} is selected, the workflow is fitted
+#'   directly without tuning.
+#'   If custom \code{tune_params} are supplied with \code{tuning_strategy = "none"},
+#'   they will be ignored with a warning.
+#' @param tuning_iterations Number of iterations for Bayesian tuning. Ignored
+#'   when \code{tuning_strategy} is not \code{"bayes"}; validation occurs only
+#'   for the Bayesian strategy.
 #' @param early_stopping Logical for early stopping in Bayesian tuning.
 #' @param adaptive Logical indicating whether to use adaptive/racing methods.
 #' @param algorithm_engines A named list specifying the engine to use for each algorithm.
@@ -42,6 +53,7 @@ train_models <- function(train_data,
                          resampling_method,
                          folds,
                          repeats,
+                         resamples = NULL,
                          tune_params,
                          metric,
                          summaryFunction = NULL,
@@ -56,6 +68,28 @@ train_models <- function(train_data,
 
   set.seed(seed)
 
+  tuning_strategy <- match.arg(tuning_strategy, c("grid", "bayes", "none"))
+
+  if (tuning_strategy == "bayes" && adaptive) {
+    warning("'adaptive' is not supported with Bayesian tuning. Setting adaptive = FALSE.")
+    adaptive <- FALSE
+  }
+
+  if (tuning_strategy == "none" && !is.null(tune_params)) {
+    warning("'tune_params' are ignored when 'tuning_strategy' is 'none'")
+  }
+
+  if (tuning_strategy == "bayes") {
+    if (!is.numeric(tuning_iterations) || length(tuning_iterations) != 1 ||
+        tuning_iterations <= 0 || tuning_iterations != as.integer(tuning_iterations)) {
+      stop("'tuning_iterations' must be a positive integer")
+    }
+  }
+
+  if (early_stopping && tuning_strategy != "bayes") {
+    message("Engine-level early stopping will be applied when supported")
+  }
+
   if (task == "classification") {
 
     if(is.null(summaryFunction)){
@@ -91,7 +125,11 @@ train_models <- function(train_data,
     metrics <- metric_set(rmse, rsq, mae)
   }
 
-  if (resampling_method == "cv") {
+  if (!is.null(resamples)) {
+    if (!inherits(resamples, "rset")) {
+      stop("'resamples' must be an 'rset' object")
+    }
+  } else if (resampling_method == "cv") {
     if (nrow(train_data) < folds) {
       stop(
         sprintf(
@@ -142,6 +180,10 @@ train_models <- function(train_data,
     stop("Unsupported resampling method.")
   }
 
+  if (use_default_tuning && is.null(resamples)) {
+    warning("Tuning is skipped because resampling is disabled")
+  }
+
   models <- list()
 
   # A helper function to choose the engine for an algorithm
@@ -205,8 +247,7 @@ train_models <- function(train_data,
         param_obj %>% dials::range_set(c(new_lb, new_ub))
       })
 
-      params_model <- params_model %>%
-        dplyr::mutate(object = if_else(id == param_name, list(updated_obj), object))
+      params_model$object[params_model$id == param_name] <- list(updated_obj)
     }
     return(params_model)
   }
@@ -229,22 +270,47 @@ train_models <- function(train_data,
     # Loop over each engine provided
     for (engine in engines) {
 
-      # Get the tuning parameters for this engine.
+      # Get default parameters for this engine
       if (use_default_tuning) {
-        engine_tune_params <- get_default_tune_params(algo,
-                                                      train_data,
-                                                      label,
-                                                      engine)
+        defaults <- get_default_tune_params(
+          algo,
+          train_data,
+          label,
+          engine
+        )
       } else {
-        engine_tune_params <- get_default_params(algo, task, num_predictors = ncol(train_data %>% dplyr::select(-!!sym(label))), engine = engine)
+        defaults <- get_default_params(
+          algo,
+          task,
+          num_predictors = ncol(train_data %>% dplyr::select(-!!sym(label))),
+          engine = engine
+        )
       }
 
-      if(algo == "logistic_reg" && engine %in% c("glm", "gee" ,"glmer" , "stan" , "stan_glmer")){
+      # User supplied tuning parameters for this algorithm/engine
+      user_params <- NULL
+      if (!is.null(tune_params) &&
+          !is.null(tune_params[[algo]]) &&
+          !is.null(tune_params[[algo]][[engine]])) {
+        user_params <- tune_params[[algo]][[engine]]
+      }
 
-        perform_tuning = FALSE
-      }else{
+      # Merge defaults with user parameters
+      engine_tune_params <- if (is.null(defaults)) list() else defaults
+      if (!is.null(user_params)) {
+        for (nm in names(user_params)) {
+          engine_tune_params[[nm]] <- user_params[[nm]]
+        }
+      }
 
-      perform_tuning <- !all(vapply(engine_tune_params, is.null, logical(1))) && !is.null(resamples)
+      if (algo == "logistic_reg" && engine %in% c("glm", "gee", "glmer", "stan", "stan_glmer")) {
+        perform_tuning <- FALSE
+      } else {
+        has_custom <- !is.null(user_params)
+        perform_tuning <- (use_default_tuning || has_custom) && !is.null(resamples)
+        if (tuning_strategy == "none") {
+          perform_tuning <- FALSE
+        }
       }
 
        # For other algorithms, use a switch that uses the current engine
@@ -286,14 +352,16 @@ train_models <- function(train_data,
                                                    train_data,
                                                    label,
                                                    tuning = perform_tuning,
-                                                   engine = engine)
+                                                   engine = engine,
+                                                   early_stopping = early_stopping)
                              },
                              "lightgbm" = {
                                define_lightgbm_spec(task,
                                                     train_data,
                                                     label,
                                                     tuning = perform_tuning,
-                                                    engine = engine)
+                                                    engine = engine,
+                                                    early_stopping = early_stopping)
                              },
                              "decision_tree" = {
                                define_decision_tree_spec(task,
